<?php
namespace Teknisa\SM\Controller;

use Teknisa\SM\Controller\SMCrud;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\SM\Service\Sm00006000ItemComposicaoServicosService as ItemComposicaoService;

class Sm00006000ItemComposicaoServicosController extends SMCrud
{
    protected $dataSourceName = "fct_itcompservico_widget";
    protected $itemComposicaoService;
    
    public function __construct ($crudService, $environment, ItemComposicaoService $itemComposicaoService) {
        parent::__construct($crudService, $environment);
        $this->itemComposicaoService = $itemComposicaoService;
    }
    
    /**
     * Salva os itens da composição.
     * 
     * @param Request $requet
     * @param Response $response
     */
    public function save(Request $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            
            if (isset($rows[0]["_cad_auto"]) && $rows[0]["_cad_auto"] == true) {
                
                $rows = $this->itemComposicaoService->substituirServicosExistentes($rows);
                foreach ($rows as $key => &$row) {
                    if (array_key_exists("NAO_INSERIR", $row)) {
                        unset($rows[$key]);
                    }
                }
                
                $request = new Request\DataSet(new DataSet("", $rows), $request->getRoutePath(), $request->getMethod(), $request->getUserId());
            }
            
            $request->setParameter('dbname', 'fct_itcompservico_widget');
            parent::save($request, $response);
            
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Exclui o serviço da composição e seus filhos.
     * 
     * @param Request $request
     * @param Response $response
     */
    public function delete(Request $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrcompservico = $row["NRCOMPSERVICO"];
            $idelemento    = $row["IDELEMENTO"];
            $nrorg         = $row["NRORG"];
            
            $this->itemComposicaoService->excluirItemComposicaoEmCascata($nrcompservico, $idelemento, $nrorg);
            $response->addMessage(new Message("Registro excluído com sucesso."));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Valida se o Elemento informado já existe na tabela de Itens da Composição.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function verificarSeElementoExiste(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $nrcompservico = $row["NRCOMPSERVICO"];
            $idelemento    = $row["IDELEMENTO"];
            
            $existe = $this->itemComposicaoService->verificarSeElementoExiste($nrcompservico, $idelemento);
            
            $response->addDataSet(new DataSet("validacao", array("existe" => $existe)));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Replica a obrigatoriedade do item da composição para seus itens filhos.
     * 
     * @param Request\Row $request
     * @param Response $response
     */
    public function replicarObrigatoriedadeItensFilhos(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $idelemento    = $row["IDELEMENTO"];
        $nrcompservico = $row["NRCOMPSERVICO"];
        $obrigatorio   = $row["IDSERVOBRIGCOMP"];
        $nrorg         = $row["NRORG"];
        
        $this->itemComposicaoService->replicarObrigatoriedadeItensFilhos($idelemento, $nrcompservico, $obrigatorio, $nrorg);
    }
    
    public function find(Request\Filter $request, Response $response) {
    
        $request->setParameter('dbname', 'fct_itcompservico_widget');
        parent::find($request, $response);
    
    }
}

?>