<?php

namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00006000ItemComposicaoServicosFactory as ItemComposicaoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class Sm00006000ItemComposicaoServicosService {
    
    protected $itemComposicaoFactory;
    protected $environment;
    protected $newCode;
    
    public function __construct(ItemComposicaoFactory $itemComposicaoFactory, Environment $environment, NewCode $newCode) {
        $this->itemComposicaoFactory = $itemComposicaoFactory;
        $this->environment = $environment;
        $this->newCode = $newCode;
    }
    
    /**
     * Procura os serviços que já existem na tabela e, se existir, seta 
     * o código no array de rows e os marca para não inserir novamente.
     * 
     * @param mixed[] $rows
     * 
     * @return mixed[]
     */
    public function substituirServicosExistentes($rows) {
        $nrorg = $this->environment->getNrOrg();
        $nrcompservico = $rows[0]["NRCOMPSERVICO"];
        $servicos = array();
        
        foreach ($rows as $row) {
            $servicos[] = $row["CDSERVORCA"];
        }
        $servicos = implode(",", $servicos);
        
        $dataset = $this->itemComposicaoFactory->getItensExistentes($nrcompservico, $servicos, $nrorg);
        
        if (count($dataset) > 0) {
            foreach ($dataset as $item) {
                for ($i = 0; $i < count($rows); $i++) {
                    if ($item["CDSERVORCA"] == $rows[$i]["CDSERVORCA"]) {
                        $rows[$i]["NRITCOMPSERVICO"] = $item["NRITCOMPSERVICO"];
                        $rows[$i]["NAO_INSERIR"] = true;
                    }
                }
            }
        }
        
        foreach($rows as &$row) {
            if (empty($row["NRITCOMPSERVICO"])) {
                $row["NRITCOMPSERVICO"] = $this->newCode->getCodeByTableName("FCT_ITCOMPSERVICO", $nrorg, 1, 10, true);
            }
        }
        
        return $rows;
    }
    
    /**
     * Exclui o serviço da composição junto com seus filhos.
     * 
     * @param integer $nritcompservico
     * @param integer $nrorg
     */
    public function excluirItemComposicaoEmCascata($nrcompservico, $idelemento, $nrorg) {
        $params = array(
            "NRORG"         => $nrorg,
            "NRCOMPSERVICO" => $nrcompservico,
            "IDELEMENTO"    => $idelemento
        );
        
        $this->itemComposicaoFactory->excluirItemComposicaoEmCascata($params);
    }
    
    /**
     * Valida se o Elemento informado já existe na tabela de Itens da Composição.
     * 
     * @param integer $nrcompservico
     * @param string $idelemento
     */
    public function verificarSeElementoExiste($nrcompservico, $idelemento) {
        $nrorg = $this->environment->getNrOrg();
        
        $params = array(
            "NRCOMPSERVICO" => $nrcompservico,
            "IDELEMENTO"    => $idelemento,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->itemComposicaoFactory->verificarSeElementoExiste($params);
        return ((int)$dataset[0]["EXISTE"] > 0);
    }
    
    /**
     * Replica a obrigatoriedade do item da composição para seus itens filhos.
     * 
     * @param string $idelemento
     * @param integer $nrcompservico
     * @param string $idservobrigcomp
     * @param integer $nrorg
     */
    public function replicarObrigatoriedadeItensFilhos($idelemento, $nrcompservico, $idservobrigcomp, $nrorg) {
        $params = array(
            "IDELEMENTO"      => $idelemento,
            "NRCOMPSERVICO"   => $nrcompservico,
            "IDSERVOBRIGCOMP" => $idservobrigcomp,
            "NRORG"           => $nrorg,
            "CDOPERULTATU"    => $this->environment->getCdOperador(),
            "NRORGULTATU"     => $this->environment->getNrOrg()
        );
        
        $this->itemComposicaoFactory->replicarObrigatoriedadeItensFilhos($params);
    }
}

?>